package controller

import (
	"github.com/gin-gonic/gin"
	"github.com/mhaya/game/game_cluster/internal/code"
	"github.com/mhaya/game/game_cluster/nodes/webadmin/common"
	"github.com/mhaya/game/game_cluster/nodes/webadmin/entity"
	"github.com/mhaya/game/game_cluster/nodes/webadmin/service"
)

type Role struct {
	sev *service.Role
}

func NewRole() *Role {
	return &Role{
		sev: service.NewRole(),
	}
}

// Add 新增角色
// @Summary 新增角色
// @Description 新增角色
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.RoleAddReq true "新增角色"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/add [post]
func (r *Role) Add(ctx *gin.Context) {
	var req entity.RoleAddReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.Add(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// List 角色列表
// @Summary 角色列表
// @Description 角色列表
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data query entity.RoleListReq true "角色列表"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/list [get]
func (r *Role) List(ctx *gin.Context) {
	var req entity.RoleListReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	resp, err := r.sev.List(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, resp)
}

// Update 修改角色
// @Summary 修改角色
// @Description 修改角色
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.RoleUpdateReq true "修改角色"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/update [post]
func (r *Role) Update(ctx *gin.Context) {
	var req entity.RoleUpdateReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.Update(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// Del 删除角色
// @Summary 删除角色
// @Description 删除角色
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data query entity.RoleDelReq true "删除角色"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/del [get]
func (r *Role) Del(ctx *gin.Context) {
	var req entity.RoleDelReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.Del(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// AddRoleAccess  添加角色权限
// @Summary 添加角色权限
// @Description 添加角色权限
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.RoleAccessAddReq true "添加角色权限"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/role_access/add [post]
func (r *Role) AddRoleAccess(ctx *gin.Context) {
	var req entity.RoleAccessAddReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.AddRoleAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// UpdateRoleAccess 修改角色权限
// @Summary 修改角色权限
// @Description 修改角色权限
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.RoleAccessUpdateReq true "修改角色权限"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/role_access/update [post]
func (r *Role) UpdateRoleAccess(ctx *gin.Context) {
	var req entity.RoleAccessUpdateReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.UpdateRoleAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// DelRoleAccess 删除角色权限
// @Summary 删除角色权限
// @Description 删除角色权限
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data query entity.RoleAccessDelReq true "删除角色权限"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/role_access/del [get]
func (r *Role) DelRoleAccess(ctx *gin.Context) {
	var req entity.RoleAccessDelReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.DelRoleAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// GetRoleAccessList 获取角色权限列表根据角色ID
// @Summary 获取角色权限列表根据角色ID
// @Description 获取角色权限列表根据角色ID
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data query entity.RoleAccessListReq true "获取角色权限列表根据角色ID"
// @Success 200 {object} []entity.AccessResp
// @Router /v1/admin/role/role_access/list [get]
func (r *Role) GetRoleAccessList(ctx *gin.Context) {
	var req entity.RoleAccessListReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	resp, err := r.sev.GetRoleAccessList(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, resp)
}

// AddAccess 添加权限路由
// @Summary 添加权限路由
// @Description 添加权限路由
// @Tags 角色-路由
// @Accept application/json
// @Produce application/json
// @Param data body entity.AccessAddReq true "添加权限路由"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/access/add [post]
func (r *Role) AddAccess(ctx *gin.Context) {
	var req entity.AccessAddReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.AddAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// DelAccess
// @Summary 删除权限路由
// @Description 删除权限路由
// @Tags 角色-路由
// @Accept application/json
// @Produce application/json
// @Param data query entity.AccessDelReq true "删除权限路由"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/access/del [Get]
func (r *Role) DelAccess(ctx *gin.Context) {
	var req entity.AccessDelReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.DelAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// UpdateAccess  修改权限路由
// @Summary 修改权限路由
// @Description 修改权限路由
// @Tags 角色-路由
// @Accept application/json
// @Produce application/json
// @Param data body entity.AccessUpdateReq true "修改权限路由"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/access/update [post]
func (r *Role) UpdateAccess(ctx *gin.Context) {
	var req entity.AccessUpdateReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.UpdateAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// ListAccess 获取权限路由列表
// @Summary 获取权限路由列表
// @Description 获取权限路由列表
// @Tags 角色-路由
// @Accept application/json
// @Produce application/json
// @Param data query entity.AccessListReq true "获取权限路由列表"
// @Success 200 {object} []entity.AccessResp
// @Router /v1/admin/role/access/list [get]
func (r *Role) ListAccess(ctx *gin.Context) {
	var req entity.AccessListReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	resp, err := r.sev.ListAccess(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, resp)
}

// GetAdminRole
// @Summary 获取该管理员的权限详情
// @Description 获取该管理员的权限详情
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data query entity.AdminBindRoleReq true "获取角色"
// @Success 200 {object} entity.AdminBindRoleResp
// @Router /v1/admin/role/admin_role_info [post]
func (r *Role) GetAdminRole(ctx *gin.Context) {
	var req entity.AdminBindRoleReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	resp, err := r.sev.GetAdminRole(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, resp)
}

// AdminBindRole 绑定角色
// @Summary 绑定角色
// @Description 绑定角色
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.AdminBindRoleReq true "绑定角色"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/admin_bind_role [post]
func (r *Role) AdminBindRole(ctx *gin.Context) {
	var req entity.AdminBindRoleReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.AdminBindRole(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}

// AdminUnBindRole 取消绑定角色
// @Summary 取消绑定角色
// @Description 取消绑定角色
// @Tags 角色
// @Accept application/json
// @Produce application/json
// @Param data body entity.AdminBindRoleReq true "取消绑定角色"
// @Success 200 {object} entity.Response
// @Router /v1/admin/role/admin_unbind_role [post]
func (r *Role) AdminUnBindRole(ctx *gin.Context) {
	var req entity.AdminBindRoleReq
	if err := ctx.ShouldBindJSON(&req); err != nil {
		common.PackOkResult(ctx, code.ParamError)
		return
	}

	err := r.sev.AdminUnBindRole(ctx, req)
	if err != nil {
		common.PackOkResult(ctx, err.Code)
		return
	}

	common.PackOkResult(ctx, code.OK, nil)
}
